# Preliminaries
##################

# Packages and libraries
library(colorspace)
library(ismev)
library(fields)

# Functions (homemade)
source("myImagePlot.R")
source("approxExtrap.R")
source("rel_change.R")

# Related to plots
pal=tim.colors
pal.qual=tim.colors



# DATA
########

# Read data
load("../data/Synthetic_nbev.RData")

# The list of thresholds to consider in the analysis
thresholds = seq(100,200,10)
ns = length(thresholds)
# List of years
years = as.numeric(rownames(nbev))
ny = length(years)
t=years
y1 = years[1]

# Clausius-Clapeyron estimate
deltaT = 1	# We assume a observed 1K warming...
Delta_cc = (1.068^deltaT-1)	# We assume a +6.8%/K   CC-relationship




# I. Average frequency 
########################


nbj_mean = apply(nbev,2,mean)

# Plot average frequencies -- Fig 7 in the paper
pdf("figures/NBJ_mean.pdf")
par(mar=c(5.1,5.1,2.1,2.1))
plot(thresholds,nbj_mean,type="l",col="red",xlab="Threshold      (mm)",ylab="Average frequency      (nb of events / year)",cex.axis=1.3,font=2,cex.lab=1.3,font.lab=2)
dev.off()





# II. Linear trends. 
#########################
# Test of H0: no change
# Note: Gaussian model is assumed

rel_ch = matrix(NA,ny,ns)	# Relative change
pv_bi = matrix(NA,ny,ns)	# Pvalue of 2-sided test
pv_uni = matrix(NA,ny,ns)	# Pvalue of 1-sided test
for (itn in 3:ny){
  tn = y1+itn-1  # End of period
  t_vect = y1:tn
  for (is in 1:ns){
    lmo = lm(nbev[1:itn,is]~t_vect)
    freq_be = lmo$fitted.values[c(1,itn)]	# Frequencies at the beginning and end
    rel_ch[itn,is] = (freq_be[2]-freq_be[1])/mean(freq_be)
    pv_bi[itn,is] = summary(lm(nbev[1:itn,is]~t_vect))$coefficients[2,4]
    ctmp = summary(lm(nbev[1:itn,is]~t_vect))$coefficients[2,]
    if (ctmp[1]>0)	pv_uni[itn,is] = pv_bi[itn,is] / 2
    else 	   	pv_uni[itn,is] = 1 - pv_bi[itn,is] / 2
  }
}

rel_ch = 100*(rel_ch-1)
rel_ch = as.data.frame(rel_ch)
rownames(rel_ch) = as.character(years)
colnames(rel_ch) = as.character(thresholds)

#pdf("Relch_NbEvt.pdf",width=7,height=5)
#myImagePlotch(t(rel_ch[10:ny,]))
#dev.off()


pv_uni = as.data.frame(pv_uni)
rownames(pv_uni) = as.character(years)
rownames(pv_bi) = as.character(years)
colnames(pv_uni) = as.character(thresholds)
colnames(pv_bi) = as.character(thresholds)

pdf("figures/Pval_lin_Fig9b.pdf",width=7,height=5)
myImagePlot(t(pv_uni[10:ny,]))
dev.off()




# III. Regression plots for selected thresholds.
##################################################
# Fig 8 (GLM) and (similar to) Fig SM3 (top panels, linear trends) of the paper

Selected_thresholds = c(100,150,200)

for (ss in Selected_thresholds){

  # Observed data + CC estimate
  yss = nbev[,paste(ss)]
  years = as.numeric(rownames(nbev))
  # Clausius Clapeyron estimate
  df_cc = approxExtrap(thresholds,nbj_mean,xout=(1+c(Delta_cc/2,-Delta_cc/2))*ss)


  ## Non-linear fit (glm)
  pdf(paste("figures/Freq",as.character(ss),"_nl_Fig8.pdf",sep=""),width=7,height=4)
  #dev.new(width=7,height=3.5)
  par(mar=c(3.1,3.1,2.1,2.1),cex.axis=1.2)
  # Observed values
  plot(years,yss,type="h",ylim=c(0,max(yss)+.5),yaxs="i",xlab="",ylab="")
  # Linear fit
  glmo = glm(yss~years,family="poisson")
  lines(years,glmo$fitted.values,lty=1,col="red")
  #require("gam")
  #gam.out=gam(yss~years,family="poisson")
  #plot(gam.out,se=T)
  # Confidence interval on regression coefficient, then plot
  glmoc = confint(glmo,level=.9)
  lines(years,exp(glmoc[2,1]*years)*mean(yss)/mean(exp(glmoc[2,1]*years)),lty=2,col="red")	# Lower bound
  lines(years,exp(glmoc[2,2]*years)*mean(yss)/mean(exp(glmoc[2,2]*years)),lty=2,col="red")	# Upper bound
  # Find the (exp-) trend which matches the CC-trend
  b = seq(-.1,.1,.001)
  fi = (exp(b*years[ny])-exp(b*years[1])) / apply(exp(b%*%t(years)),1,"mean")
  cst = (df_cc[2]-df_cc[1])/mean(yss)
  b_cc = b[which.min(abs(fi-cst))]
  f_cc_nl = exp(b_cc*years)*mean(yss)/mean(exp(b_cc*years))
  lines(years,f_cc_nl,lty=5,col="green")	## CC-relationship
  dev.off()
}





# IV. GLM/Poisson model
#########################
# Test of H0: no change
# Figure 9  in the paper

pvp_bi = matrix(NA,ny,ns)
pvp_uni = matrix(NA,ny,ns)
it1 = which(t==y1)
for (itn in 3:ny){
  tn = y1+itn-1  # End of period
  t_vect = y1:tn
  for (is in 1:ns){
    glm_fit = glm(nbev[it1:itn,is]~t_vect,family="poisson")
    pvp_bi[itn,is] = summary(glm_fit)$coefficients[2,4]
    ctmp = summary(glm_fit)$coefficients[2,1]
    if (ctmp>0)	pvp_uni[itn,is] = pvp_bi[itn,is] / 2
    else 	pvp_uni[itn,is] = 1 - pvp_bi[itn,is] / 2
  }
}

pvp_uni = as.data.frame(pvp_uni)
rownames(pvp_uni) = as.character(t_vect)
rownames(pvp_bi) = as.character(t_vect)
colnames(pvp_uni) = as.character(thresholds)
colnames(pvp_bi) = as.character(thresholds)
pdf("figures/Pval_Fig9.pdf",width=7,height=5)
myImagePlot(t(pvp_uni[10:ny,]))
dev.off()





# V. Plot Delta_freq and Delta_freq_rel over the entire period
###############################################################
# Figures 9 and SM2 (bottom panels) in the paper

L = rel_change(nbev,rel=F,method="poisson",Delta_cc=Delta_cc,yref=y1)
LR = rel_change(nbev,method="poisson",Delta_cc=Delta_cc,yref=y1)

# Plots
 # Warning : plot with customized y-axis for relative frequency change...
 ys = c(-.5,-.2,0,.2,.5,1,2,4)
 nys = length(ys)
 xs = 1:nys
 ss = smooth.spline(xs,ys)
 ss_ref = predict(ss,x=seq(1,nys,.1))

sign_str = function(x){
	r = as.character(x)
	r[x>0] = "+"
	r[x<0] = "-"
	r[x==0] = ""
	return(r)
}

  pdf("figures/Delta_freq.pdf")
  par(cex=1.4,mar=c(6,4,4,2),mgp=c(2.2,.8,0))
  plot(thresholds,L[,1],type="l",col="red",ylab="",xlab="Threshold    (mm)",ylim=range(L[,1:4]))
  lines(thresholds,L[,2],col="red",lty=2)
  lines(thresholds,L[,3],col="red",lty=2)
  lines(thresholds,L[,4],col="green",lty=5)

  tLR = LR
  #ylim_tLR = range(LR)
  ylim_tLR = c(-.5,4.5)
  par(cex=1.4,mar=c(6,4,4,2),mgp=c(2.2,.8,0))
  plot(thresholds,tLR[,1],type="l",col="red",yaxt="n",ylab="",ylim=ylim_tLR,xlab="")
  title(xlab="Threshold    (mm)")
  xtLR = ceiling(ylim_tLR[1]):floor(ylim_tLR[2])
  ylab_tLR = paste(sign_str(xtLR),as.character(100*xtLR),"%",sep="")
  axis(2,at=xtLR,label=ylab_tLR,las=2)
  lines(thresholds,tLR[,2],col="red",lty=2)
  lines(thresholds,tLR[,3],col="red",lty=2)
  lines(thresholds,tLR[,4],col="green",lty=5)

  dev.off()



